"""Functions for calculations on rolling windows."""
import math

import numpy as np
from numpy import typing as npt


def rolling_mean(x, w_vl, undef_val=np.nan):
    """
    Compute rolling mean of an array `x` over a window of length `w_vl`.

    The convention is that in the returned array of values, a value is
    shifted to the left with respect to a data window for which it (the value)
    is computed, i.e. the first element of the returned array corresponds
    to sample mean of `x[:w_vl]`.
    Indices of the returned array for which a rolling statistic is undefined are
    hence located at the array's tail; their values are defined by parameter
    `undef_val` and by default set to numpy.nan.

    :param x: (numpy.ndarray) 1-dimensional array
    :param w_vl: (int) A numeric scalar. Length of `x` window over which a
    rolling statistics is computed.
    :param undef_val: Scalar. Value inserted into a tail of returned array,
    for which a rolling statistic is undefined. Default is numpy.nan.
    :return: (numpy.ndarray) An array of rolling statistic values. The length of
        output array equals the length of `x` array.
    """
    out = np.convolve(x, np.ones(w_vl) / w_vl, mode="valid")
    return np.concatenate((out, [undef_val] * (w_vl - 1)))


def rolling_diff1(x, undef_val=np.nan):
    """
    Compute rolling difference between the subsequent elements of the array `x`.

    :param x: (numpy.ndarray) 1-dimensional array
    :param undef_val: Scalar. Value inserted into a tail of returned array,
    for which a rolling statistic is undefined. Default is numpy.nan.
    :return: (numpy.ndarray) An array of rolling difference values. The length of
        output array equals the length of `x` array.
    """
    # Note for defining the filter vec:
    # It must be [1, -1] b/c numpy.convolve implementation uses
    # *reversed* filtered signal in contrary to classical notation of
    # convolution.
    # Hence by using [1,-1] we obtain the desired effect:
    # "A positive derivative means that the function is increasing".
    filter_vec: npt.NDArray[np.int_] = np.array([1, -1])
    out = np.convolve(x, filter_vec, mode="valid")
    return np.concatenate((out, [undef_val] * 1))


def rolling_corr(x, y, undef_val=np.nan):
    """
    Compute rolling correlation between an array `x` and a short array `y`.

    The used convention is that in the returned array of values, a value is
    shifted to the left with repect to a data window for which it (the value)
    is computed, i.e. the first element of the returned array corresponds
    to sample correlation between `x[:len(y)]` and `y`.
    Indices of the returned array for which a rolling statistic is undefined are
    hence located at the vector's tail; their values are defined by parameter
    `undef_val` and by default set to numpy.nan.

    :param x: (numpy.ndarray) 1-dimensional array
    :param y: (numpy.ndarray) 1-dimensional array, shorter than `x`.
    :param undef_val: Scalar. Value inserted into a tail of returned array,
    for which a rolling statistic is undefined. Default is numpy.nan.
    :return: (numpy.ndarray) An array of rolling statistic values. The length of
        output array equals the length of `x` array.
    """
    w_vl = len(y)
    # Fix for the fact that numpy reverses the shorter array. Reference:
    # https://stackoverflow.com/questions/47096025/the-result-of-numpy-convolve-is-not-as-expected?noredirect=1&lq=1
    y_REV = y[::-1]
    # Mean of x (rolling mean), mean of y
    win = np.ones(w_vl)
    meanx = np.convolve(x, win / w_vl, mode="valid")
    meany = np.mean(y_REV)
    # Unbiased estimator of sample covariance
    covxy = (np.convolve(x, y_REV, mode="valid") - w_vl * meanx * meany) / (w_vl - 1)
    # Unbiased estimator of sample variance
    # S^2 = \frac{\sum X^2 - \frac{(\sum X)^2}{N}}{N-1}
    sigmax2: npt.NDArray[np.float_] = (  # type: ignore
        np.convolve(x**2, win, mode="valid")
        - ((np.convolve(x, win, mode="valid")) ** 2) / w_vl
    ) / (w_vl - 1)

    # Correct numerical errors if any
    sigmax2[sigmax2 < 0] = 0
    # Unbiased estimator of sample standard deviation
    # Note "w_vl / (w_vl-1)" gives version of vector variance consistent i.e. with R
    sigmax = np.sqrt(sigmax2)
    sigmay = np.sqrt(np.var(y) * w_vl / (w_vl - 1))
    # Compute unbiased estimator of sample covariance
    # * Surpress the warning which may occur if sigmax or sigmay being numerical 0
    # * Surpress the warning which occurs b/c of tail of vectors being filled with np.nan
    with np.errstate(divide="ignore", invalid="ignore"):
        corxy = covxy / (sigmax * sigmay)
    # Correct for infinite values which may occur if
    # sigmax or sigmay being numerical 0
    corxy[~np.isfinite(corxy)] = 0

    return np.concatenate((corxy, [undef_val] * (w_vl - 1)))


def rolling_smooth(x, w, x_fs=1, replace_undef_with_closest_val=True):
    """
    Compute moving window average of a time-series `x`.

    The used convention is that in the returned array of values, a value is
    centered with respect to a data window for which it (the value) is computed.
    Indices of the returned array for which a rolling smooth is undefined are
    hence located at the vector's head and tail; if
    `replace_undef_with_closest_val = False`,
    their values are set to  NAs (numpy.nan) whereas if
    `replace_undef_with_closest_val = True`, their values are with first and last
    non-NA value, respectively.

    Time-series frequency `x_fs` and a length of a moving window (expressed in
    seconds) `w` together determine `w_vl = round(w * x_fs)`, a length of a
    moving window expressed in a vector length (i.e., number of vector indices).

    Note:
    - `w_vl` must be equal or greater than 1. Otherwise, an error is risen.
    - `w_vl` must not be longer than length of `x` vector.  Otherwise, an error
    is risen.
    - If `w_vl` is an even number then `(w_vl - 1)` value is silently used
    instead.

    :param x: (numpy.ndarray) 1-dimensional array. A time-series for which a
    moving window average is computed.
    :param w: (float) A numeric scalar. A length of a moving window, expressed in
    time (seconds).
    :param x_fs: (float) A numeric scalar. Sampling frequency of `x`, expressed
    in Hz.
    :returns: (numpy.ndarray): An array of rolling smooth values. The length of
    output array equals the length of `x` vector.
    """
    n = len(x)
    # Check validity of smoothing window width `w` given frequency `x_fs`
    # of `x` data collection
    w_vl = int(round(w * x_fs))
    if w_vl < 1:
        raise ValueError(
            "Averaging window vector length `w_vl`"
            "(refer to function's details description) must not be "
            "less than 3. Define wider `w` averaging "
            "window length"
        )
    if w_vl > n:
        raise ValueError(
            "Averaging window vector length `w_vl`"
            "(refer to function's details description) must be less "
            "than `x` vector length. Define narrower `w` averaging window "
            "length"
        )
    # Replace `w_vl` with closest odd integer no larger than `w_vl`
    w_vl = w_vl + (w_vl % 2) - 1
    # Compute length of a moving window's "wing"
    w_vl_wing = int(math.floor(w_vl / 2))
    # Compute rolling mean (convention: result values values are shifted to the left
    # with repect to a data window over which mean is computed; hence undefined
    # rolling mean values are put as NA's at the end of the resulted vector)
    vec_out0 = rolling_mean(x, w_vl)
    # Shift rolling mean values (new convention: result values values are centered
    # with repect to a data window over which mean is computed; hence undefined
    # rolling mean values are put as NA's at both head and tail of the resulted vector)
    vec_out0 = vec_out0[: (n - 2 * w_vl_wing)]
    vec_out = [np.nan] * n
    vec_out[w_vl_wing : (w_vl_wing + len(vec_out0))] = vec_out0
    # If specified, replace NA values in the returned vector in head and tail with
    # first and last non-NA value
    if replace_undef_with_closest_val:
        # vec_out[:w_vl_wing] = [vec_out[w_vl_wing]] * w_vl_wing
        # vec_out[(n - w_vl_wing):] = [vec_out[(n - w_vl_wing - 1)]] * w_vl_wing
        vec_out[:w_vl_wing] = x[:w_vl_wing]
        vec_out[(n - w_vl_wing) :] = x[(n - w_vl_wing) :]

    return np.array(vec_out)



"""Computes information on wear time.

Wear time is defined as corresponding to time of stride start for each element of
`segment_pattern` function output.

The wear time information is NOT used for walking strides/bouts identificaton,
as I (@MK) assumed the waling bouts identification algorithm works in a way
it will return info there is no walking for the non-wear time (b/c of, i.e.,
vector magnitude filetering params which set some minimal threshold that
non-wear data wound not pass).

Instead, this script is used to compute wear data for the vizualization purposes,
that is, for computing heatmap matrices used to show per-subject walking
distribution across days of acc collection for Dublin study.
(Mea culpa for params hard-coded on the top of the file.)
"""
import os
import sys
from typing import List

import numpy as np
from actigrapy.algorithms.wear_detection.wear_data.wear_data import WearData
from numpy import typing as npt

# This script contains function which computes information on wear time
#
# PARAMS

project_dir_path = "/home/karasma6/PROJECTS/walking_segmentation"
data_CP_dir_path = "/CHBS/data/digital_rwe/data_CP"
# Directory of pattern segmentation results, per-subject and per-dad files
results_partial_dir = os.path.join(
    project_dir_path, "data_results/20190805_DDV001A2302/results_partial"
)
# Desired directory of device wear information that corresponds to
# pattern segmentation results
results_wear_dir = os.path.join(
    project_dir_path, "data_results/20190805_DDV001A2302/results_wear"
)
# Path to directory with Dublin wear data HDF5 files
dublin_hdf5_wear_dir = os.path.join(
    data_CP_dir_path,
    "AnalysisOutput/DDV001/A2302/ggir_wearDetection/51ac5b63f48302fc0ca02e99066ed740",
)


def wrapper_compute_results_wear(subj_fname_prefix):
    """Compute information on wear time."""
    # List of parial results (pattern segmentation results) for the patient
    res_fname_list = [
        val for val in os.listdir(results_partial_dir) if subj_fname_prefix in val
    ]

    for res_fname in res_fname_list:
        print("res_fname: " + res_fname)

        try:
            res_f = np.load(os.path.join(results_partial_dir, res_fname))
            res_f_dad = int((res_fname.split(".")[0]).split("_")[3])

            # Path to file with HDF5 wearDetection data
            hdf5_wear_fname = (
                "DDV001A2302" + "_" + res_fname.split("_")[1] + ".wearDetection"
            )
            hdf5_wear_fpath = os.path.join(dublin_hdf5_wear_dir, hdf5_wear_fname)

            # Pull wear data
            n_samples = int(np.max(res_f[:, 0])) + 1
            wear_data = WearData(hdf5_wear_fpath)
            [daily_wear, time] = wear_data.get_data(
                what="wear",
                when={"type": "dad24", "number": [res_f_dad]},
                output_frequency=30,
                n_samples=n_samples,
            )
            # Subset wear data information to what is needed
            is_worn: List[int] = [int(daily_wear[int(val)]) for val in res_f[:, 0]]
            is_worn_arr: npt.NDArray[np.int_] = np.array(is_worn)
            print("len(is_worn_arr) = " + str(len(is_worn_arr)))

            # Save array to file
            np.save(os.path.join(results_wear_dir, res_fname), is_worn_arr)

        except Exception as e:
            print("Error: " + str(e))
            continue


if __name__ == "__main__":
    wrapper_compute_results_wear(sys.argv[1])



"""Determine whether a segment is part of a walking bout."""
from copy import copy

import numpy as np
from numpy import typing as npt

from pyadept.sliding_functions import rolling_mean, rolling_diff1
from pyadept.segment_pattern import rank_chunks_of_ones


def segment_walking_bouts(
    spdesc_df,
    sim_MIN_stride=0.85,
    sim_MIM_bridge=0.75,
    dur_MIN=0.85,
    dur_MAX=1.4,
    ptp_r_MIN=0.4,
    ptp_r_MAX=3.0,
    vmc_r_MIN=0.1,
    vmc_r_MAX=0.8,
    mean_abs_diff_med_p_MAX=0.2,
    mean_abs_diff_med_t_MAX=0.2,
    mean_abs_diff_dur_MAX=0.25,
    keep_valid_walking_bout_only=True,
):
    """
    Determine whether a segment is part of a walking bout.

    Takes pandas.DataFrame with stride pattern
    segmentation results (output of `describe_segments` function after conversion
    to pandas.DataFrame) and for each stride identified,
    determines whether it is a part of walking instance / walking bout.

    Conceptual description of the procedure:
    Step 1: For each stride in pattern segmentaton results, determine if
    individual stride is a "VALID" STRIDE (takes filtering of its duration,
    vector magnitude values etx.)
    Step 2: Identify WALKING INSTANCES, that is, sequences of >=3 "valid"
    subsequent walking strides. It checking whether or not those subsequent
    walking strides are "similar" to each other within each walking instance.
    Step 3: Identify WALKING BOUTS, that is, sequences subsequent walking strides
    which are consisting og >= 1 WALKING INSTANCES. This step is to merge
    identified walking instances which are i.e. very close to each other and
    it is plausible to think they come from one walking action, but something
    in a middle of that walking action has missed the "valid" stride criteria.
    3 cases of conditions upon which the merging happens:
    (1) no break between two neighbouring walking instances,
    or 1-element break (bridge) between the two neighbouring walking instances
    (2) 2-element break (bridge) between the two neighbouring walking instances,
    where at least 1 element of the bridge is close to passing valid stride condition
    (3) 3-element break (bridge) between the two neighbouring walking instances,
    where at least 2 elements of the bridge are close to passing valid stride condition.

    Each identified walking instance receives `'walking_instance_id'`,
    and each identified walking bout receives `'walking_bout_id'`.

    The primary conceptual difference is that while I assume strides from same
    WALKING BOUT are coming from one walking bout i.e. for bout size computation,
    I only want to use strides
    from WALKING INSTANCE (that is, strides, which have `'walking_instance_id'`
    assigned) for any analytics purposes as those are which we determined as
    "VALID" STRIDES.

    :param spdesc_df: (pandas.DataFrame) Data frame with stride pattern
    segmentation results. Output of `describe_segments` function
    (after conversion to pandas.DataFrame). Data frame columns:
    "tau_idx",
    "dur_vl",
    "sim",
    "med_x",
    "med_y",
    "med_z",
    "med_p",
    "med_t",
    "med_r",
    "std_x",
    "std_y",
    "std_z",
    "std_p",
    "std_t",
    "std_r",
    "ptp_x",
    "ptp_y",
    "ptp_z",
    "ptp_p",
    "ptp_t",
    "ptp_r",
    "vmc_r",
    "dur".
    :param sim_MIN_stride: (float) Scalar. Step 1: valid strides filtering parameter.
    Determines minimum similarity value between predefined stride pattern and
    identified stride occurrence needed to consider it as an individual "valid"
    stride.
    :param sim_MIM_bridge: (float) Scalar. Step 3: merging into walking bout parameter.
    Determines minimum similarity value between predefined stride pattern and
    identified stride occurrence needed to consider a stride as a one which lies
    in-between two walking instances.
    :param dur_MIN: (float) Scalar. Step 1: valid strides filtering parameter.
    Determines minimum stride duration time needed to consider it as an individual
    "valid" stride. Expressed in seconds.
    :param dur_MAX: (float) Scalar. Step 1: valid strides filtering parameter.
    Determines maximum stride duration time needed to
    consider corresponding segmented stride as an individual "valid" stride.
    Expressed in seconds.
    :param ptp_r_MIN: (float) Scalar. Step 1: valid strides filtering parameter.
    Determines minimum peak-to-peak value for vector magnitude [r] needed to
    consider corresponding segmented stride as an individual "valid" stride.
    :param ptp_r_MAX: (float) Scalar. Step 1: valid strides filtering parameter.
    Determines maximum peak-to-peak value for vector magnitude [r] needed to
    consider corresponding segmented stride as an individual "valid" stride.
    :param vmc_r_MIN: (float) Scalar. Step 1: valid strides filtering parameter.
    Determines minimum vector magnitude count value needed to
    consider corresponding segmented stride as an individual "valid" stride.
    :param vmc_r_MAX: (float) Scalar. Step 1: valid strides filtering parameter.
    Determines maximum vector magnitude count value needed to
    consider corresponding segmented stride as an individual "valid" stride.
    :param mean_abs_diff_med_p_MAX: (float) Scalar. Step 2: merging into walking instance parameter.
    Maximum value of mean of absolute differences in Azimuth median for 3 subsequent
    valid strides needed to consider them as valid walking instance.
    :param mean_abs_diff_med_t_MAX: (float) Scalar. Step 2: merging into walking instance parameter.
    Maximum value of mean of absolute differences in Elevation median for 3 subsequent
    valid strides needed to consider them as valid walking instance.
    :param mean_abs_diff_dur_MAX: (float) Scalar. Step 2: merging into walking instance parameter.
    Maximum value of mean of absolute differences in duration time for 3 subsequent
    valid strides needed to consider them as valid walking instance.
    :param keep_valid_walking_bout_only: (boolean) Whether or not to return
    data frame with strids which are a part of any valid walking bout, or
    return the whole imput table of all stride pattern segmentation results
    (with additional labels of whether or not a segmented stride is a part of any
    walking instance and walking bout).
    :return: (pandas.DataFrame) Data frame with stride pattern
    segmentation results (output of `describe_segments` function) extended
    by two additional columns: `'walking_instance_id'` and `'walking_bout_id'`.
    """
    # Define condition for a segmented pattern to be a potential WALKING STRIDE
    cond_stride_0 = (
        (spdesc_df["ptp_r"] >= ptp_r_MIN)
        & (spdesc_df["ptp_r"] <= ptp_r_MAX)
        & (spdesc_df["vmc_r"] >= vmc_r_MIN)
        & (spdesc_df["vmc_r"] <= vmc_r_MAX)
        & (spdesc_df["dur"] >= dur_MIN)
        & (spdesc_df["dur"] <= dur_MAX)
    )
    cond_stride: npt.NDArray[np.int_] = np.array(
        cond_stride_0 & (spdesc_df["sim"] >= sim_MIN_stride)).astype("int")
    cond_stride_bridge: npt.NDArray[np.int_] = np.array(
        cond_stride_0 & (spdesc_df["sim"] >= sim_MIM_bridge)
    ).astype("int")

    # Define conditions for a potential WALKING STRIDE to be a part of a WALKING INSTANCE
    cond_stride_mean3 = rolling_mean(np.array(cond_stride), w_vl=3, undef_val=0) > 0.9
    med_p_diff1 = np.abs(rolling_diff1(np.array(spdesc_df["med_p"]), undef_val=0))
    med_t_diff1 = np.abs(rolling_diff1(np.array(spdesc_df["med_t"]), undef_val=0))
    dur_diff1 = np.abs(rolling_diff1(np.array(spdesc_df["dur"]), undef_val=0))
    cond_med_p_diff1_mean2 = (
        rolling_mean(med_p_diff1, w_vl=2, undef_val=mean_abs_diff_med_p_MAX)
        < mean_abs_diff_med_p_MAX
    )
    cond_med_t_diff1_mean2 = (
        rolling_mean(med_t_diff1, w_vl=2, undef_val=mean_abs_diff_med_t_MAX)
        < mean_abs_diff_med_t_MAX
    )
    cond_dur_diff1_mean2 = (
        rolling_mean(dur_diff1, w_vl=2, undef_val=mean_abs_diff_dur_MAX)
        < mean_abs_diff_dur_MAX
    )
    cond_wi = (
        cond_stride_mean3
        & cond_med_p_diff1_mean2
        & cond_med_t_diff1_mean2
        & cond_dur_diff1_mean2
    )

    # Define walking instance (wi) ID
    wi_id0 = rank_chunks_of_ones(cond_wi)
    wi_id_concat: npt.NDArray[np.float_] = np.column_stack((wi_id0, np.roll(wi_id0, 1),
                                                            np.roll(wi_id0, 2)))
    wi_id_concat[np.isnan(wi_id_concat)] = -1
    wi_id = np.nanmax(wi_id_concat, axis=1)
    wi_id[wi_id < 0] = np.nan
    # Define walking bout (wb) ID
    wi_id_CPY = copy(wi_id)
    wb_id = copy(wi_id)
    wi_id_unq = np.sort(np.unique(wi_id[~np.isnan(wi_id)]))
    for i in range(len(wi_id_unq) - 1):
        # Current walking instance
        id_unq = wi_id_unq[i]
        id_unq_idxMAX = np.max(np.where(wi_id_CPY == id_unq)[0])
        # Next walking instance
        id_unq_next = wi_id_unq[i + 1]
        id_unq_next_idx = np.where(wi_id_CPY == id_unq_next)[0]
        id_unq_next_idxMIN = np.min(id_unq_next_idx)
        id_unq_next_idxMAX = np.max(id_unq_next_idx)
        # no break or 1-element break between the two meighbouring walking instances
        bridge_len = id_unq_next_idxMIN - id_unq_idxMAX - 1
        cond_join_1 = bridge_len <= 1
        # 2-element break break between the two meighbouring walking instances
        cond_join_2 = (bridge_len == 2) & (
            np.mean(cond_stride_bridge[(id_unq_idxMAX + 1) : (id_unq_next_idxMIN)]) > 0
        )
        # 3-element break break between the two meighbouring walking instances
        cond_join_3 = (bridge_len == 3) & (
            np.mean(cond_stride_bridge[(id_unq_idxMAX + 1) : (id_unq_next_idxMIN)])
            > 0.5
        )
        # If any of the conditions is matched, join the two meighbouring walking
        # instances onto one walking bout
        if cond_join_1 | cond_join_2 | cond_join_3:
            # Update walking bouts
            wb_id[id_unq_idxMAX : (id_unq_next_idxMAX + 1)] = id_unq
            # Update walking instances
            wi_id_CPY[id_unq_next_idxMIN : (id_unq_next_idxMAX + 1)] = id_unq
            wi_id_unq[i + 1] = wi_id_unq[i]

    # Assign columns to pandas data frame
    spdesc_df["walking_instance_id"] = wi_id
    spdesc_df["walking_bout_id"] = wb_id
    # Keep only those for which walking bout is not np.nan
    if keep_valid_walking_bout_only:
        spdesc_df = spdesc_df[~np.isnan(spdesc_df["walking_bout_id"])]
    # Return Pandas dataframe
    return spdesc_df



"""Core algorithm for stride pattern segmentation."""
from copy import copy
from typing import List

import numpy as np
import pandas as pd
from numpy import typing as npt

from pyadept.sliding_functions import rolling_corr, rolling_smooth


def dstack_product(x, y):
    """Compute a 2D array whose rows consists of combination of all pairs of elements.

    :param x: (numpy.array) An arry.
    :param y: (numpy.array) An arry.
    :return: A 2D array.
    """
    return np.dstack(np.meshgrid(x, y)).reshape(-1, 2)


def interp_vec(vec, vl_out):
    """Interpolate array linearly to have certain length.

    :param vec: (numpy.ndarray) An array.
    :param vl_out: (int) A scalar. Number of elements in resulted array.
    :return: (numpy.ndarray) An array.
    """
    x = np.linspace(0, 1, len(vec))
    y = vec
    xvals = np.linspace(0, 1, vl_out)
    yinterp = np.interp(xvals, x, y)
    return yinterp


def interp_scale_vec(vec, vl_out):
    """Interpolate array to have certain length.

    The values are then normalized to have mean 0, variance 1.

    A multiplier sqrt((vl_out-1) / vl_out) is used in the scaling
    function so as the resulted array of values has variance equal 1
    according to R convention.

    :param vec: (numpy.ndarray) An array.
    :param vl_out: (int) A numeric scalar. Number of elements in resulted array.
    :return: (numpy.ndarray) An array.
    """
    x = np.linspace(0, 1, len(vec))
    y = vec
    xvals = np.linspace(0, 1, vl_out)
    yinterp = np.interp(xvals, x, y)
    yinterps = (
        (yinterp - np.mean(yinterp))
        * (1 / np.std(yinterp))
        * np.sqrt((vl_out - 1) / vl_out)
    )
    return yinterps


def stride_templ_df_to_array(
    df_path, sensor_location_id_val="left_wrist", collection_id_val="size_3"
):
    """
    Read precomputed stride pattern templates from CSV file to 2D numpy array.

    For detailed description of origin and content of the precomputed stride
    pattern templates, see `stride_template` description in R adeptdata package
    documentation:
    https://cran.r-project.org/web/packages/adeptdata/adeptdata.pdf.

    :param templ_df_path: (str) String scalar. Absolute path to data frame with
    precomputed stride pattern templates.
    :param sensor_location_id_val: (str) String scalar. Sensor location for
    which the pattern templates are to be returned. One of:
    'left_wrist', 'left_hip', 'left_ankle', 'right_ankle'.

    :param collection_id_val: (str) String scalar. Denotes number of unique
    location-specific pattern templates which are to be returned.  One of:
    'size_1', 'size_2', 'size_3', 'size_4', 'size_5'.
    :return: (numpy.ndarray) 2D array with stride pattern templates. Each row
    of the array corresponds to one distinct pattern template.
    """
    # Read CSV
    templ_df = pd.read_csv(df_path)
    templ_df_sub = templ_df[
        (templ_df["sensor_location_id"] == sensor_location_id_val)
        & (templ_df["collection_id"] == collection_id_val)
    ]
    templ_df_cols_sub = [col for col in templ_df_sub.columns if "ph" in col]
    templ_df_sub = templ_df_sub[templ_df_cols_sub]
    if templ_df_sub.empty:
        raise ValueError(
            "Templates data frame is empty. "
            "Select correct `sensor_location_id_val` and/or `collection_id_val` values. "
            "Correct `sensor_location_id_val` values: 'left_wrist', 'left_hip', "
            "'left_ankle', 'right_ankle'. Correct `collection_id` values: 'size_1', "
            "'size_2', 'size_3', 'size_4', 'size_5'. "
        )
    templ_array = templ_df_sub.values
    return templ_array


def rank_chunks_of_ones(arr):
    """Label individual chunks of subsequent `1`'s in numpy.array.

    Reference:
    https://stackoverflow.com/questions/57573350/fast-python-ish-way-of-ranking-chunks-of-1s-in-numpy-array

    :param arr: (numpy.array) An array.
    :return: (numpy.array) An array which contains distinct labels (int scalars)
    for distinct individual chunks of subsequent `1`'s. The elements corresponding
    to places where `arr` does not have `1` value are set to numpy.nan.
    """
    m = np.r_[False, arr.astype(bool), False]
    idx = np.flatnonzero(m[:-1] != m[1:])
    ll = idx[1::2] - idx[::2]
    out: npt.NDArray[np.float_] = np.full(len(arr), np.nan, dtype=float)
    out[arr != 0] = np.repeat(np.arange(len(ll)), ll)
    return out


def do_ftune(tau_i, T_i, x_ftune, nbhwing_vl, interp_vl_min, interp_vl_max):
    """Finetune."""
    tau_1 = tau_i
    tau_2 = tau_1 + T_i - 1
    tau_1_nbh: npt.NDArray[np.float_] = np.array(
        range(max(0, tau_1 - nbhwing_vl), min(tau_2, tau_1 + nbhwing_vl + 1))
    )
    tau_2_nbh: npt.NDArray[np.float_] = np.array(
        range(max(0, tau_2 - nbhwing_vl), min(len(x_ftune), tau_2 + nbhwing_vl + 1))
    )
    # Mask to assure the potential updated pattern location corresponds to a
    # pattern which duration is within allowed pattern duration range
    tau_prod: npt.NDArray[np.float_] = dstack_product(tau_2_nbh, tau_1_nbh)
    tau_mask: npt.NDArray[np.float_] = tau_prod[:, 0] - tau_prod[:, 1] + 1
    tau_mask = (tau_mask >= interp_vl_min) & (tau_mask <= interp_vl_max)
    # Identify fine-tuned location which corresponds to signal magnitude peaks
    nbh_x_prod = dstack_product(x_ftune[tau_2_nbh], x_ftune[tau_1_nbh])
    # @MK: changed 2019-07-11
    # nbh_x_multip = (nbh_x_prod[:, 0] * nbh_x_prod[:, 1]) * tau_mask
    nbh_x_multip = nbh_x_prod[:, 0] * nbh_x_prod[:, 1]
    nbh_x_multip[~tau_mask] = np.nan
    amax = np.nanargmax(nbh_x_multip)
    tau_2_new, tau_1_new = tau_prod[amax, :]
    # Update x_ftune to mark location of newly identified pattern
    x_ftune[(tau_1_new + 1) : tau_2_new] = np.nan
    return [tau_1_new, tau_2_new - tau_1_new + 1, x_ftune]


def segment_pattern(
    x,
    x_fs,
    templ_array,
    pattern_dur_grid,
    x_sim_smooth_w=np.nan,
    ftune=False,
    x_ftune_nbh_w=np.nan,
    x_ftune_smooth_w=np.nan,
):
    """
    Segment stride patterns from a 1-dimensional data array `x`.

    Algorithm uses an array of pattern templates to iteratively identify in
    the array `x` occurrences of the pattern. Fine-tuning algorithm is available
    to exploit the fact that beginning and end of a walking stride is (in many
    use cases) characterized by peak of magnitude of vector magnitude array `x`,
    and hence local maxima detection allows to identify those beginning/end
    precisely.

    :param x: (numpy.ndarray) 1-dimensional array. For typical application of
    strides identification from accelerometry data, this would be [r], that is,
    vector magnitude of raw [x,y,z] data.
    :param x_fs: (float) A numeric scalar. Sampling frequency of `x`, expressed
    in Hz.
    :param templ_array: (numpy.ndarray) 2D array with stride pattern templates,
    where each distinct pattern takes 1 row of the 2D array.
    :param pattern_dur_grid: (list of float scalars) Grid of potential
    pattern durations (durations the algorithm searches within), expressed
    in seconds.
    :param x_sim_smooth_w: (float) Scalar. Width (expressed in seconds) of
    smoothing window used to smooth accelerometry data VM for
    similarity computation between VM and pattern template(s).
    :param ftune: (boolean) Scalar. Whether or not to use fine-tune procedure
    (see the description above).
    :param x_ftune_nbh_w: (float) Scalar. Width (expressed in seconds) of data
    window ("neighbourhood") centered at preliminarily identified location
    of stride start/end, where the search of local maxima happends in fine-tune
    procedure (see: param `ftune`).
    :param x_ftune_smooth_w: (float) Scalar. Width (expressed in seconds) of
    smoothing window used to smooth accelerometry data VM for fine-tune procedure.
    :return: List of lists that describes the pattern segmentation results.
    Each "inner" list is a 3-element-long list
    describing one individual segmented stride:
    0. index of `x` VM time-series, where is the start of identified stride,
    1. duration (expressed of number of indices of `x`) of the identified stride,
    2. similarity (correlation) between the identified stride and stride pattern
    used in segmentation.
    """
    # Compute potential pattern vector length values
    pattern_vl_grid0 = np.sort(
        np.unique((np.array(pattern_dur_grid) * x_fs).astype(int))
    )
    vl_grid_min = np.min(np.ceil(np.array(pattern_dur_grid) * x_fs).astype(int))
    vl_grid_max = np.max(np.floor(np.array(pattern_dur_grid) * x_fs).astype(int))
    pattern_vl_grid = pattern_vl_grid0[
        (pattern_vl_grid0 >= vl_grid_min) & (pattern_vl_grid0 <= vl_grid_max)
    ]
    vl_grid_n = len(pattern_vl_grid)
    x_n = len(x)
    # Sanity checks
    if max(pattern_vl_grid) > x_n:
        raise ValueError(
            "Vector length of maximum potential pattern duration is longer than `x` vector length. "
            "Cannot identify a pattern that long. Decrease max value in `pattern_dur_grid`."
        )
    if min(pattern_vl_grid) < 3:
        raise ValueError(
            "Vector length of minimum potential pattern duration is less than 3. "
            "Cannot identify a pattern that short. Increase min value in `pattern_dur_grid`."
        )
    # Compute list of 2D arrays of rescaled pattern template(s)
    # Each 2D array corresponds to particular potential pattern vector length
    templ_rescaled = []
    for vl_i in pattern_vl_grid:
        item_tmp: npt.NDArray[np.float_] = np.stack(
            [interp_scale_vec(templ_i, vl_i) for templ_i in templ_array]
        )
        templ_rescaled.append(item_tmp)
    # Smooth x for computation of similarity matrix between x and template(s)
    x_sim = (
        copy(x) if np.isnan(x_sim_smooth_w) else rolling_smooth(x, x_sim_smooth_w, x_fs)
    )
    # Compute similarity matrix between x and template(s)
    simmat: npt.NDArray[np.float_] = np.stack(
        [
            np.amax(
                np.stack(
                    [rolling_corr(x_sim, templ_j) for templ_j in templ_rescaled_i]
                ),
                axis=0,
            )
            for templ_rescaled_i in templ_rescaled
        ]
    )
    simmat[np.isnan(simmat)] = -3
    # Define objects needed for fine-tuning of pattern location, if finetuning is selected
    x_ftune, nbhwing_vl, interp_vl_min, interp_vl_max = [None] * 4
    if ftune:
        # Smooth x for pattern location
        x_ftune = (
            copy(x)
            if np.isnan(x_ftune_smooth_w)
            else rolling_smooth(x, x_ftune_smooth_w, x_fs)
        )
        # Define neighbourhood within which we fine-tune pattern locations
        x_ftune_nbh_vl = int(x_ftune_nbh_w * x_fs)
        interp_vl_min = min(pattern_vl_grid)
        interp_vl_max = max(pattern_vl_grid)
        nbhwing_vl = min(int(np.floor(x_ftune_nbh_vl / 2)), interp_vl_min - 1)
    # Identify pattern occurrences iteratively
    out = []
    while True:
        # Break loop if all nan in similarity matrix
        # (:= no room is left for any more pattern in x signal)
        if (simmat < -2).all():
            break
        # Identify pattern location via finding similarity matrix maximum
        simmat_maxrow, simmat_maxcol = np.unravel_index(simmat.argmax(), simmat.shape)
        tau_i = simmat_maxcol
        T_i = pattern_vl_grid[simmat_maxrow]
        sim_i = simmat[simmat_maxrow][simmat_maxcol]
        # Fine-tune pattern location to match maxima of x data vector magnitude
        if ftune:
            tau_i, T_i, x_ftune = do_ftune(
                tau_i, T_i, x_ftune, nbhwing_vl, interp_vl_min, interp_vl_max
            )
        out_tmp = [tau_i, T_i, sim_i]
        out.append(out_tmp)
        # Update similarity matrix to mark location of newly identified pattern
        for i in range(0, vl_grid_n):
            col_idx_min = max(0, tau_i - pattern_vl_grid[i] + 2)
            col_idx_max = min(
                tau_i + T_i - 2, x_n - 2
            )  # x_n-2 (instead x_n-1)to prevent from escaping outside
            simmat[i : i + 1, col_idx_min : col_idx_max + 1] = -3
    return out


def segment_pattern_wrap(x_slice_vl, x, **kwargs):
    """Wrap segment_pattern function.

    Its role is to cut the (potentially very long one) vector magnitude accelerometry
    signal into shorter pieces, apply `segment_pattern` algorithm at each of those
    pieces, and merge the results.

    :param x_slice_vl: (int) Scalar. Length of a short arrays into which
    the `x` vector magnitude time-series is cut into.
    :param x:
    :param kwargs: The following arguments of `segment_pattern` function:
    x_fs, templ_array, pattern_dur_grid, x_sim_smooth_w, ftune, x_ftune_nbh_w, x_ftune_smooth_w
    :return: List of lists that describes the pattern segmentation results.
    Each "inner" list is a 3-element-long list
    describing one individual segmented stride:
    0. index of `x` VM time-series, where is the start of identified stride,
    1. duration (expressed of number of indices of `x`) of the identified stride,
    2. similarity (correlation) between the identified stride and stride pattern
    used in segmentation.
    """
    # Segment pattern from signal at once
    if np.isnan(x_slice_vl):
        return segment_pattern(x, **kwargs)
    # Segment pattern from signal at once over
    # partitions of signal x onto slices
    x_nn = len(x)
    out: List[List[float]] = []
    x_slice_idx0 = list(np.arange(0, x_nn, x_slice_vl).astype(int))
    x_slice_idx0 = x_slice_idx0 if x_slice_idx0[-1] == x_nn else x_slice_idx0 + [x_nn]
    for i in range(0, len(x_slice_idx0) - 1):
        idx_1 = x_slice_idx0[i]
        idx_n = x_slice_idx0[i + 1]
        x_slice = x[idx_1:idx_n]
        out_x_slice0 = segment_pattern(x_slice, **kwargs)
        out_x_slice = [[row[0] + idx_1] + row[1:] for row in out_x_slice0]
        out = out + out_x_slice
    return out



"""Run `segment_pattern` and `describe_segments`."""

import os
from copy import copy

import numpy as np
from actigrapy.actigraphy_data.activity_data import RawActivityData

# Hard-coded (thought to be always fixed for it) segmentation params,
# likely to be read from some external params file later.
from pyadept.describe_segments import describe_segments
from pyadept.segment_pattern import segment_pattern_wrap
from pyadept.segment_pattern import stride_templ_df_to_array

x_slice_vl = 6000
x_sim_smooth_w = 0.2
ftune = True
x_ftune_nbh_w = 0.6
x_ftune_smooth_w = np.nan
pattern_dur_grid = np.linspace(0.5, 4, 30, endpoint=True)


def wrapper_identify_walking(hdf5_fpath, templ_fpath, results_partial_dir):
    """
    Run the first, most consuming part of walking identification algorithm.

    This function runs `segment_pattern` and `describe_segments` parts
    (`segment_walking_bouts` is not done).

    It process the whole HDF5 data for one patient.
    It saves the results of this part of walking identification algorithm as
    "partial results" that is, subject- and dad24-specific numpy.array files.

    Identifying walking bouts based on output of this file takes ~1s per
    subject- and dad24-specific numpy.array file. There is potential
    value (use case) in storing the pattern segmentation results first,
    hence this is what this wrapper does.

    :param hdf5_fpath: Path to subject-specific HDF5 file.
    :param templ_fpath: Path to CSV file containing precmputed stride pattern
    templates. The current location on the cluster is:
    "/home/karasma6/PROJECTS/walking_segmentation/data/stride_templates.csv"
    :param results_partial_dir: Path to directory where "partial results" are
    saved as subject- and dad24-specific numpy.array files.
    :return: none
    """
    # Get wrist stride pattern template
    # templ_fpath = os.path.join(project_dir, 'data', 'stride_templates.csv')
    templ_array = stride_templ_df_to_array(
        templ_fpath, sensor_location_id_val="left_wrist", collection_id_val="size_3"
    )
    print("shape templ_array: " + str(templ_array.shape))

    # Define prefix of partial results file names
    results_partial_fname_prefix = os.path.basename(hdf5_fpath).split(".")[0]

    # Read HDSF5 file
    raw_activity_data = RawActivityData(hdf5_fpath)
    print("raw_activity_data DONE")
    # Get number of dad24
    dad24_cnt = raw_activity_data.get_dad_count()
    # Get sampling frequency
    data_fs = int(raw_activity_data.get_sampling_frequency())

    for dad24_i in range(0, dad24_cnt):
        try:
            # Pull raw data
            [data_xyzptr, data_time] = raw_activity_data.get_data(
                do_correct=False,
                what="xyzptr",
                when={"type": "dad24", "number": [dad24_i]},
            )

            # Run pattern segmentation algorithm
            sp_out = segment_pattern_wrap(
                x_slice_vl=x_slice_vl,
                x=copy(data_xyzptr[:, 5]),
                x_fs=data_fs,
                templ_array=templ_array,
                pattern_dur_grid=pattern_dur_grid,
                x_sim_smooth_w=x_sim_smooth_w,
                ftune=ftune,
                x_ftune_nbh_w=x_ftune_nbh_w,
                x_ftune_smooth_w=x_ftune_smooth_w,
            )

            # Describe segments
            spdesc_out = describe_segments(sp_out, data_xyzptr, data_fs)

            # Save results to file
            results_partial_fname = (
                    results_partial_fname_prefix + "_dad24_" + str(dad24_i) + ".npy"
            )
            results_partial_fpath = os.path.join(
                results_partial_dir, results_partial_fname
            )
            np.save(results_partial_fpath, np.array(spdesc_out))
            print("Saved results to: " + results_partial_fpath)

        except Exception as e:
            print("Error:" + str(e))
            continue



"""Provide statistics about segments."""
import numpy as np


def vmc(vec):
    """
    Compute Vector Magnitude Count of a time-series.

    :param vec:
    :return:
    """
    return np.mean(np.abs(vec - np.mean(vec)))


def describe_segments(sp_out, data_xyzptr, data_fs):
    """
    Provide statistics (median, std, peak-to-peak, vector magnitude count).

    Statistics are provided for each individual pattern occurrence segmented with
    `segment_pattern` function.

    Uses the output of `segment_pattern` function and
    raw `xyzptr` data from which the data values corresponding to each identified
    pattern occurrence are pulled from.

    :param sp_out: (list of lists) Output of `segment_pattern` function. Each
    "inner list" constsis of 3 elements:
    0. pattern occurrence start location index,
    1. pattern occurrence length expressed in array length,
    2. correlation value between the accelerometry data and predefined pattern template.
    :param data_xyzptr: (numpy.array) "xyzptr" raw accelerometry data array.
    :param data_fs: (float) Scalar. Accelerometry data collection frequency,
    expressed in Hz.
    :return: List of lists. Each "inner" list describes one segmented pattern
    occurrence, and contains 23 elements - numerical summaries of a segmented pattern
    occurrence. Those 23 summaries are:
    0. pattern occurrence start location index,
    1. pattern occurrence length expressed in array length,
    2. correlation value between the accelerometry data and predefined pattern template,
    3. median of `x` Cartesian coordinate of corresponding accelerometry data segment,
    4. median of `y` Cartesian coordinate of corresponding accelerometry data segment,
    5. median of `z` Cartesian coordinate of corresponding accelerometry data segment,
    6. median of `p` spherical coordinate of corresponding accelerometry data segment,
    7. median of `t` spherical coordinate of corresponding accelerometry data segment,
    8. median of `r` spherical coordinate of corresponding accelerometry data segment,
    9. standard deviation of `x` Cartesian coordinate of corresponding accelerometry data segment,
    10. standard deviation of `y` Cartesian coordinate of corresponding accelerometry data segment,
    11. standard deviation of `z` Cartesian coordinate of corresponding accelerometry data segment,
    12. standard deviation of `p` spherical coordinate of corresponding accelerometry data segment,
    13. standard deviation of `t` spherical coordinate of corresponding accelerometry data segment,
    14. standard deviation of `r` spherical coordinate of corresponding accelerometry data segment,
    15. peak-to-peak of `x` Cartesian coordinate of corresponding accelerometry data segment,
    16. peak-to-peak of `y` Cartesian coordinate of corresponding accelerometry data segment,
    17. peak-to-peak of `z` Cartesian coordinate of corresponding accelerometry data segment,
    18. peak-to-peak of `p` spherical coordinate of corresponding accelerometry data segment,
    19. peak-to-peak of `t` spherical coordinate of corresponding accelerometry data segment,
    20. peak-to-peak of `r` spherical coordinate of corresponding accelerometry data segment,
    21. Vector Magnitude Count,
    22. duration (expressed in seconds).
    """
    desc_out = [
        out_i
        # median
        + [np.median(data_xyzptr[(out_i[0]) :
                                 (out_i[0] + out_i[1]), i]) for i in range(6)]
        # std
        + [np.std(data_xyzptr[(out_i[0]) : (out_i[0] + out_i[1]), i]) for i in range(6)]
        # ptp
        + [np.ptp(data_xyzptr[(out_i[0]) : (out_i[0] + out_i[1]), i]) for i in range(6)]
        # VMC
        + [vmc(data_xyzptr[(out_i[0]) : (out_i[0] + out_i[1]), 5])]
        # stride duration
        + [out_i[1] / data_fs]
        for out_i in sp_out
    ]
    return desc_out



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyadept']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.22.2,<2.0.0', 'pandas>=1.4.1,<2.0.0']

setup_kwargs = {
    'name': 'pyadept',
    'version': '0.1.1',
    'description': 'A python implementation of',
    'long_description': '# adept algorithm -- Python implementation\n\nThis repo contains Python implementation of ADEPT method -- a fast, scalable, and accurate method for pattern segmentation in time series -- and, additionally, its extension for  walking segmentation from free-living wrist-worn sensor accelerometry data. \n\nRelated publoshed work: \n\n1. Karas, M., Straczkiewicz, M., Fadel, W., Harezlak, J., Crainiceanu, C. M., & Urbanek, J.\nK. (2019). Adaptive empirical pattern transformation (ADEPT) with application to walking\nstride segmentation. Biostatistics, 22(2), 331–347.\n[https://doi.org/10.1093/biostatistics/kxz033](https://doi.org/10.1093/biostatistics/kxz033 )\n\n2. Karas, M., Urbanek, J. K. U., Illiano, V. P., Bogaarts, G., Crainiceanu, C. M., Dorn, J. F.\n(2021). Estimation of free-living walking cadence from wrist-worn sensor accelerometry\ndata and its association with SF-36 quality of life scores. Physiological measurement,\n42(6). [https://doi.org/10.1088/1361-6579/ac067b](https://doi.org/10.1088/1361-6579/ac067b)\n\nThe R implementation is available in R package adept ([CRAN index](https://cran.r-project.org/web/packages/adept/index.html)): \n\n- `segmentPattern()` -- ADEPT method implementation (paper 1 above), \n- `segmentWalking()` -- ADEPT extension for  walking segmentation from free-living wrist-worn sensor accelerometry data (paper 2 above). \n\nThis Python implementation presented in this repo was developed by Marta Karas during her internship at Novartis @ Sensor Data Analytics Team in summer 2019 (IP not proteceted). \n\n# Overview of walking segmentation algorithm\n\nThe proposed algorithm for walking segmentation from free-living wrist-worn sensor accelerometry data consists of 3 main parts: \n\n### Step (1): Stride pattern segmentation from raw accelerometry data. \n\n- Input: raw vector magnitude accelerometry data, stride pattern templates, other segmentation parameters.  \n- Output: List of lists, where each "inner list" has 3 elements which describe one identified data segment: \n\n    - 0. index of the start of the segment, \n    - 1. duration of the segment, expressed in array length, \n    - 2. correlation value between raw vector magnitude accelerometry data and stride pattern template. </br></br>\n\n- Related implementations: \n  - `python/segment_pattern.py` -- <span style="color:magenta">**ADEPT method implementation (paper 1 above); corresponds to R\'s `adept::segmentPattern()`**</span>\n  - `python/segment_pattern_utils.py`\n  - `python/sliding_functions.py`</br></br>\n  \n- Corresponding tests:  \n  - `python/test_segment_pattern.py`\n  - `python/test_sliding_functions.py`\n\n### Step (2): Describing each data segment obtained in Step 1. with numeric summaries of corresponding raw accelerometry data.\n\n- Input: step (1) output, raw accelerometry data. \n\n- Output: List of lists, where each "inner list" has 23 elements which describe one identified data segment. \n\n- Related implementations: \n\n  - `python/describe_segments.py`\n\n### Step (3). Identification of: (a) valid walking strides, (b) walking instances, (c) walking bouts. \n\n- Input: step (2) output, further algorithm parameters. \n\n- Output: Data frame, where each row represents one identified data segment; it has columns `\'walking_instance_id\'` and `\'walking_bout_id\'` which define walking instance and walking bout ID, respectively. There is an option to return data frame of all identified data segments contained in step (2) output, or only those if they correspond to some identified walking bout. \n\n- Related implementations: \n\n  - `python/segment_walking_bouts.py` -- <span style="color:magenta">**ADEPT extension for  walking segmentation from free-living wrist-worn sensor accelerometry data (paper 2 above); corresponds to R\'s `adept::segmentWalking()`**</span>\n\n### Explanation: (a) valid walking stride, (b) walking instance, (c) walking bout\n\nDefinitions: \n\n- valid walking stride - identified data segment which passes certain filtering conditions; parameters of those conditions are arguments of `python/segment_walking_bouts.py` function.\n\n- walking instance - a sequence of >=3 subsequent valid walking strides. To be a valid walking instance, those >=3 subsequent strides must also be "similar" to each other in a sense that their duration, azimuth angle median, elevation angle median are not larger (in their mean among 3 subsequent strides) than some parameter values; those parameters are arguments of `python/segment_walking_bouts.py` function.\n\n- walking bout -  walking instance, possibly joined with another walking instance. We merge walking instances which are i.e. very close to each other and it is plausible to think they come from one walking action, but, say, something in a middle of that walking action has missed the "valid" stride criteria.  3 cases of conditions upon which the merging happens: \n\n    - (case 1) no break between two neighbouring walking instances, or 1-element break (bridge) between the two neighbouring walking instances, \n    - (case 2) 2-element break (bridge) between the two neighbouring walking instances, where at least 1 element of the bridge is close to passing valid stride condition; \n    - (case 3) 3-element break (bridge) between the two neighbouring walking instances, where at least 2 elements of the bridge are close to passing valid stride condition. The primary conceptual difference is that information of walking bout may be used i.e. to compute walking bout size, but only the strides from walking instance (subset of walking bout) are the ones one may want to use in analysis etc. \n\n### Time execution of the above steps implementation\n\n- Step (1) is the most consuming one and takes 1-2 minutes per-subject per-dad24 with fs = 30 Hz of data collection. \n- Step (3) is the least time consuming and takes 1-2 seconds per-subject per-dad24 with fs = 30 Hz of data collection. Step (2) is also level of a few seconds. \n\n###  Wrapper for the above steps implementation\n\nA wrapper code: `python/wrapper_identify_walking.py` computes Step (1) and Step (2) of the algorithm steps above and save final results to file (per subject-, per dad24-specific numpy array files). I meant it to be something close to wrapper in the pipeline, but ran out of time to prepare it to be closer. \n\n- Input: 3 arguments (HDF4 file path, pattern template file path, partial results dir path)\n- Output: Step (2) output saved to file (precisely, per subject-, per dad24-specific numpy array files). \n',
    'author': 'Marta Karass',
    'author_email': 'mkaras@hsph.harvard.edu',
    'maintainer': 'Ali Neishabouri',
    'maintainer_email': 'ali.neishabouri@theactigraph.com',
    'url': 'https://github.com/actigraph/pyadept',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)
